/*
//  iTMTeXEngineManager.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Tue Oct 02 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSException.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSBundle.h>
#import <Foundation/NSNotification.h>
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSFileManager.h>
#import <Foundation/NSEnumerator.h>
#import <AppKit/NSApplication.h>
#import "NSString(iTeXMac).h"
#import "iTMTeXEngineManager.h"

NSString * const iTMEnginesKey = @"Engines";
NSString * const iTMFormatsKey = @"Formats";
NSString * const iTMRelativePathKey = @"Relative Path";
NSString * const iTMInputKey = @"Input";
NSString * const iTMOutputsKey = @"Outputs";
NSString * const iTMMLTeXKey = @"MLTeX";
NSString * const iTMTranslateKey = @"Translate";
NSString * const iTMInteractionKey = @"Interaction";

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMTeXEngineManager
/*"This object implements the data access to a resource where typeset engines are described. All data are stored in the Engines.plist dictionary available inside the main bundle. Each available engine is the key of a dictionary object. This dictionary  contains an array for key "Formats" listing all formats available for this engine (tex and etex do not allow the same formats). When appropriate, this dictionary should contain BOOL's for keys "ML", "Translate" and "Interaction". The "ML" key corresponds to MLTeX, the "Translate" key to the translate table of tex, and the "Interaction" to the one of tex.
The "Input" key value is an integer and describes the format of the input data: TeX = 0, DVI = 1, PS = 2, PDF = 3. The "Outputs" key is an array of integers describing the format of the outputs. For example, pdftex declares 0 in input and a 1 length array containing only "3" in outputs.
No consistency test is done and if the data file points to unreachable engine, iTeXMac will not complain until the engine is really used. As it is external ressources, it provides great extensiveness."*/
@implementation iTMTeXEngineManager
static iTMTeXEngineManager * _iTMTeXEngineManager = nil;
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= sharedTeXEngineManager
+ (id) sharedTeXEngineManager;
/*"Lazy initialisation.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _iTMTeXEngineManager? _iTMTeXEngineManager:
        (_iTMTeXEngineManager = [[self allocWithZone: [NSApp zone]] init]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= init
- (id) init;
/*"The first object inited is the shared one.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(_iTMTeXEngineManager)
    {
        if(![self isEqual: _iTMTeXEngineManager])
            [super dealloc];
        return [_iTMTeXEngineManager retain];
    }
    else
    {
        if(self = [super init])
        {
            [self->_Engines autorelease];
            NS_DURING
            self->_Engines = [[NSDictionary alloc] initWithContentsOfFile:
                    [[NSBundle bundleForClass: [self class]] pathForResource: @"TeX Engines" ofType: [NSString string]]];
            NS_HANDLER
            self->_Engines = nil;
            NSLog(@"-[iTeXMacTeXEngineManager init] problem with bundle ressource TeX Engines");
            NS_ENDHANDLER
        }
        return _iTMTeXEngineManager = self;
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= dealloc
- (void) dealloc;
/*"Never dealloc.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self retain];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= engines
- (NSArray *) engines;
/*"Returns all keys of the #{engines}.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id result = [_Engines objectForKey: iTMEnginesKey];
    return [result isKindOfClass: [NSArray class]]? result: nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= engineForKey:
- (NSDictionary *) engineForKey: (NSString *) anEngine;
/*"Returns the array of the #{engines} for key the engine given by the %{compileDescriptor}.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id result = [_Engines objectForKey: anEngine];
    return [result isKindOfClass: [NSDictionary class]]? result: nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= formatsForEngine:
- (NSArray *) formatsForEngine: (NSString *) anEngine;
/*"Returns the array of the #{engines} for key the engine given by the %{compileDescriptor}.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id result = [[self engineForKey: anEngine] objectForKey: iTMFormatsKey];
    return [result isKindOfClass: [NSArray class]]? result: nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= outputsForEngine:
- (NSArray *) outputsForEngine: (NSString *) anEngine;
/*"Returns the array of the #{engines} for key the engine given by the %{compileDescriptor}.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id result = [[self engineForKey: anEngine] objectForKey: iTMOutputsKey];
    return [result isKindOfClass: [NSArray class]]? result: nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= inputForEngine:
- (int) inputForEngine: (NSString *) anEngine;
/*"Returns the array of the #{engines} for key the engine given by the %{compileDescriptor}.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id result = [[self engineForKey: anEngine] objectForKey: iTMInputKey];
    return [result respondsToSelector: @selector(intValue)]? [result intValue]: kiTMEngineIOTeXFormat;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= pathForEngine:
- (NSString *) pathForEngine: (NSString *) anEngine;
/*"Returns the array of the #{engines} for key the engine given by the %{compileDescriptor}.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id result = [[self engineForKey: anEngine] objectForKey: iTMRelativePathKey];
    return [result isKindOfClass: [NSString class]]? result: [NSString string];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= BOOLGetter()
#define BOOLGetter(getter, key)\
- (BOOL) getter: (NSString *) anEngine;\
{\
    id result = [[self engineForKey: anEngine] objectForKey: iTMRelativePathKey];\
    return [result respondsToSelector: @selector(boolValue)]? [result boolValue]: NO;\
}
BOOLGetter(engineUsesInteraction, iTMInteractionKey);
BOOLGetter(engineUsesTranslate, iTMTranslateKey);
BOOLGetter(engineUsesMLTeX, iTMMLTeXKey);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= outputFileFormatGetter()
#define outputFileFormatGetter(getter, k)\
- (BOOL) getter: (NSString *) anEngine;\
{\
    BOOL result;\
    NS_DURING\
    NSEnumerator * enumerator = [[self outputsForEngine: anEngine] objectEnumerator];\
    NSNumber * number;\
    while(number = [enumerator nextObject])\
        if(result = ([number intValue] == k))\
            break;\
    NS_HANDLER\
    [NSApp reportException: localException];\
    result = NO;\
    NS_ENDHANDLER;\
    return result;\
}
outputFileFormatGetter(engineHasTeXOutput, kiTMEngineIOTeXFormat);
outputFileFormatGetter(engineHasDVIOutput, kiTMEngineIODVIFormat);
outputFileFormatGetter(engineHasPSOutput, kiTMEngineIOPSFormat);
outputFileFormatGetter(engineHasPDFOutput, kiTMEngineIOPDFFormat);
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMTeXEngineManager

