/*
//  iTMTextStorage.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Tue Oct 16 2001.
//  Copyright  2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSString.h>
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSAttributedString.h>
#import <Foundation/NSException.h>
#import <Foundation/NSEnumerator.h>
#import <Foundation/NSCharacterSet.h>
#import <Foundation/NSNotification.h>
#import <Foundation/NSUserDefaults.h>
#import <Foundation/NSArchiver.h>
#import <AppKit/NSColor.h>
#import <AppKit/NSFont.h>
#import <AppKit/NSFontManager.h>
#import <AppKit/NSLayoutManager.h>
#import <AppKit/NSApplication.h>
#import <AppKit/NSTextView.h>
#import "NSUserDefaults(iTeXMac).h"
#import "NSString(iTeXMac).h"
#import "iTMTextStorage.h"
//#import "iTMTextStorage(iTMAnchors).h"
#import "iTMSyntaxParser.h"
#import "iTMLiteScanner.h"
#import "iTMEncodingController.h"
#import "iTMTextWatcher.h"
#import "iTMSettings.h"

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMTextStorage
/*"Semi abstract class that defines a text storage and fixes attributes with respect to some syntax, more precisely TeX syntax and Log files syntax. Concrete classes are iTMTeXTextStorage with iTMTeXSyntaxParser, with iTMLogSyntaxParser."*/
@implementation iTMTextStorage
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initialize
+ (void) initialize;
/*"Initialize the encoding controller.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing
"*/
{
    [super initialize];
    [iTMEncodingController initialize];//Critical: forces intialization
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= syntaxParserClass
+ (Class) syntaxParserClass;
/*"This is the one of the three initializers available!!! More precisely, the first one.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing
"*/
{
    return [iTMSyntaxParser class];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= init
- (id) init;
/*"Designated initializer.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing
"*/
{
    if(self = [super init])
    {
        [_MAS autorelease];
        _MAS = [[NSMutableAttributedString alloc] init];
        [self setStringEncoding: 0];
//        [self initAnchorsControllers];
        [self initSyntaxParser];
        _InvalidAttributesRange = NSMakeRange(0, 0);
    }
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initWithString:attributes:
- (id) initWithString: (NSString *) aString attributes: (NSDictionary *) attributes;
/*"Designated initializer.
When no syntax parser is given, the layout is not invalidated and the attributes are not fixed.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing
"*/
{
    if(self = [super init])
    {
        if(!aString) aString = [NSString string];
        [_MAS autorelease];
        _MAS = [[NSMutableAttributedString alloc] initWithString: aString];
        [self setStringEncoding: 0];
        [self initSyntaxParser];
        _InvalidAttributesRange = NSMakeRange(0, 0);
//        [self initAnchorsControllers];
    }
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initWithEncodedStringData:
- (id) initWithEncodedStringData: (NSData *) docData;
/*"This is the one of the three initializers available!!! More precisely, the third one.
When no syntax parser is given, the layout is not invalidated and the attributes are not fixed.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: BEWARE of unicode data beginning: not implemented.
"*/
{
    NSString * string;
    NSStringEncoding usedEncoding;
    int prefix = 0;
    // testing for unicode encoding
    if([docData length]>1 && ((prefix = [docData deserializeIntAtIndex: 0]), (prefix == 0xFFFE)) || (prefix == 0xFEFF))
    {
        usedEncoding = NSUnicodeStringEncoding;
        string = [[NSString alloc] initWithData: docData encoding: usedEncoding];
    }
    else if([[NSUserDefaults standardUserDefaults] boolForKey: iTMIsAutoStringEncodingKey])
    {
        NSStringEncoding automaticEncoding;
        NSStringEncoding preferredEncoding = [[NSUserDefaults standardUserDefaults]
                                                        integerForKey: iTMPreferredStringEncodingKey];
        NSString * headerEncoding;
        string = [[NSString alloc] initWithData: docData encoding: preferredEncoding];
        headerEncoding = [string stringForCommentedKey: iTMEncodingHeaderKey
                                    forRange: NSMakeRange(0, 0) effectiveRange: nil inHeader: YES];
        _iTMiTSFlags.canSetStringEncoding = ([headerEncoding length] > 0? 0: 1);

        automaticEncoding = [[iTMEncodingController
                                    stringEncodingFromString: headerEncoding] unsignedIntValue];
        // accepted encodings are those for which NSString gives a non void localizedNameOfStringEncoding...
        if(automaticEncoding && [[NSString localizedNameOfStringEncoding: automaticEncoding] length] &&
            (automaticEncoding != preferredEncoding))
        {
            [string release];
            string = [[NSString alloc] initWithData: docData encoding: automaticEncoding];
            usedEncoding = automaticEncoding;
        }
        else
        {
            usedEncoding = preferredEncoding;
        }
    }
    else
    {
        usedEncoding = [[NSUserDefaults standardUserDefaults] integerForKey: iTMOpenStringEncodingKey];
        if(!usedEncoding)
            usedEncoding = [[NSUserDefaults standardUserDefaults] integerForKey: iTMPreferredStringEncodingKey];
        NSAssert(usedEncoding>0, @"Broken code: default default encoding missing.");//
        string = [[NSString alloc] initWithData: docData encoding: usedEncoding];
        _iTMiTSFlags.canSetStringEncoding = 1;
    }
    self = [self initWithString: (string? string: [NSString string]) attributes: nil];
    [self setStringEncoding: usedEncoding];
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= encodedStringDataAllowLossyConversion
- (NSData *) encodedStringDataAllowLossyConversion: (BOOL) flag;
/*"This is the one of the three initializers available!!! More precisely, the third one.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing
"*/
{
    NSStringEncoding stringEncoding = [self dataStringEncoding];
    NSLog(@"Writing with %@ string encoding.", [NSString localizedNameOfStringEncoding: stringEncoding]);
    return [[[self string] stringByUsingLineEndings: [self lineEndingsUsed]]
					dataUsingEncoding: stringEncoding allowLossyConversion: flag];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= invalidateLayout
- (void) invalidateLayout;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSEnumerator * enumerator = [[self layoutManagers] objectEnumerator];
    NSLayoutManager * LM;
    NSRange range = NSMakeRange(0, [[self string] length]);
    while(LM = [enumerator nextObject])
	[LM invalidateLayoutForCharacterRange: range isSoft: NO actualCharacterRange: nil];
    NSLog(@"invalidateLayout");
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= dealloc
- (void) dealloc;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [_MAS autorelease];
    _MAS = nil;
    [self setSyntaxParser: nil];
    #if 0
    [self setInputAnchorsController: nil];
    [self setMarkAnchorsController: nil];
    [self setSectionAnchorsController: nil];
    [self setLabelAnchorsController: nil];
    #endif
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initSyntaxParser
- (void) initSyntaxParser;
/*"Lazy initializer, subclasses will override this message to return their own default syntax parser.
This implementation returns the shared iTMSyntaxParser;
Version History: jerome.laurens@u-bourgogne.fr (03/08/02)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self replaceSyntaxParser: [[[self class] syntaxParserClass] syntaxParser]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= syntaxParser
- (id) syntaxParser;
/*"Lazy initializer, subclasses will override this message to return their own default syntax parser.
This implementation returns the shared iTMSyntaxParser;
Version History: jerome.laurens@u-bourgogne.fr (03/08/02)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _SP;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= replaceSyntaxParser:
- (void) replaceSyntaxParser: (id) argument;
/*"Sends a #{replaceTextStorage} to its syntax parser.
Version History: jerome.laurens@u-bourgogne.fr (11/07/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setSyntaxParser: argument];
    [[self syntaxParser] replaceTextStorage: self];
    [self cleanAttributesInRange: NSMakeRange(0, [self length])];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setSyntaxParser:
- (void) setSyntaxParser: (id) argument;
/*"Use the above method instead.
Version History: jerome.laurens@u-bourgogne.fr (11/07/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [iTMSyntaxParser class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] iTMSyntaxParser argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    else
    {
        [_SP autorelease];
        _SP = [argument retain];
        [_SP setTextStorage: self];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= mutableAttributedString
- (NSMutableAttributedString *) mutableAttributedString;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _MAS;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= string
- (NSString *) string;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [_MAS string];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= fixesAttributesLazily
- (BOOL) fixesAttributesLazily;
/*"YES.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= isITMCommentAtIndex:
- (BOOL) isITMCommentAtIndex: (unsigned) aLocation;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self syntaxParser] isITMCommentAtIndex: aLocation];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= groupRangeAtIndex:
- (NSRange) groupRangeAtIndex: (unsigned) index;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self string] groupRangeAtIndex: index];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= lineEndingsUsed
- (unsigned int) lineEndingsUsed;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(!_iTMiTSFlags.lineEndingsUsed)
        _iTMiTSFlags.lineEndingsUsed = [iTMEncodingController lineEndingsUsedInDocument: self] % 8;
    return _iTMiTSFlags.lineEndingsUsed;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setLineEndingsUsed:
- (void) setLineEndingsUsed: (unsigned int) argument;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    _iTMiTSFlags.lineEndingsUsed = argument % 8;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= stringEncoding
- (NSStringEncoding) stringEncoding;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _OpenStringEncoding;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= dataStringEncoding
- (NSStringEncoding) dataStringEncoding;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSStringEncoding stringEncoding = [[NSUserDefaults standardUserDefaults] boolForKey: iTMIsAutoStringEncodingKey]?
                            [[iTMEncodingController stringEncodingFromString:
                                [[self string] stringForCommentedKey: iTMEncodingHeaderKey
                        forRange: NSMakeRange(0, 0) effectiveRange: nil inHeader: YES]] unsignedIntValue]:
                                                [self stringEncoding];
    if(!stringEncoding) stringEncoding = [self stringEncoding];
    if(!stringEncoding) stringEncoding = [[NSUserDefaults standardUserDefaults]
							integerForKey:iTMPreferredStringEncodingKey];
    return stringEncoding;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setStringEncoding:
- (void) setStringEncoding: (NSStringEncoding) argument;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
   _OpenStringEncoding = argument;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= canSetStringEncoding
- (BOOL) canSetStringEncoding;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _iTMiTSFlags.canSetStringEncoding;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= processEditing
- (void) processEditing;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: cache some information...
"*/
{
    [super processEditing];
    _iTMiTSFlags.canSetStringEncoding = [[[self string] stringForCommentedKey: iTMEncodingHeaderKey
                                forRange: NSMakeRange(0, 0) effectiveRange: nil inHeader: YES] length]? 0: 1;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= attributesAtIndex:effectiveRange:
- (NSDictionary *) attributesAtIndex: (unsigned) aLocation effectiveRange: (NSRangePointer) aRangePtr;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self ensureAttributesAreFixedInRange: NSMakeRange(aLocation, 1)];
    return [[self syntaxParser] attributesAtIndex: aLocation effectiveRange: aRangePtr];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= attributesAtIndex:longestEffectiveRange:inRange:
- (NSDictionary *) attributesAtIndex: (unsigned) aLocation longestEffectiveRange: (NSRangePointer) aRangePtr inRange: (NSRange) rangeLimit;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self ensureAttributesAreFixedInRange: NSMakeRange(aLocation, 1)];
    return [[self syntaxParser] attributesAtIndex: aLocation longestEffectiveRange: aRangePtr inRange: rangeLimit];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= attribute:atIndex:longestEffectiveRange:inRange:
- (NSDictionary *) attribute: (NSString *) name atIndex: (unsigned) aLocation longestEffectiveRange: (NSRangePointer) aRangePtr inRange: (NSRange) rangeLimit;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self ensureAttributesAreFixedInRange: NSMakeRange(aLocation, 1)];
    return [[self syntaxParser] attribute: name atIndex: aLocation longestEffectiveRange: aRangePtr inRange: rangeLimit];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setAttributes:range:
- (void) setAttributes: (NSDictionary *) attributes range: (NSRange) aRange;
/*"Does nothing: no one should change the attributes except the receiver itself.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    NSLog(@"someone has asked..., %@", attributes);
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= replaceCharactersInRange:withString:
- (void) replaceCharactersInRange: (NSRange) aRange withString: (NSString *) aString;
/*"Relies on the shared syntax parser. May invalidate some range of characters if it was not easy or rapid to parse
the text for correct syntax coloring.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSRange R = NSMakeRange(0, 0);
    [[self syntaxParser] replaceCharactersInRange: aRange withString: aString invalidRange: &R];
    if(R.length)
        if(_InvalidAttributesRange.length)
            _InvalidAttributesRange = NSUnionRange(_InvalidAttributesRange,
                NSIntersectionRange(R, NSMakeRange(0, [selflength])));
        else
            _InvalidAttributesRange = R;
    if(_InvalidAttributesRange.length)
        [self invalidateAttributesInRange: _InvalidAttributesRange];
//    NSLog(@"replaceCharactersInRange: %@", NSStringFromRange(_InvalidAttributesRange));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= refixAttributes
- (void) refixAttributes;
/*"Parse the whole text again to fix the attributes, ignoring the valid ranges of attributes.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    _InvalidAttributesRange = NSMakeRange(0, [self length]);
    [self invalidateLayout];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= cleanAttributesInRange:
- (void) cleanAttributesInRange: (NSRange) aRange;
/*"Fix the attributes in the invalid part of the specified range.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    aRange = NSIntersectionRange(aRange, NSMakeRange(0, [self length]));
    if(aRange.length)
        if(_InvalidAttributesRange.length)
            _InvalidAttributesRange = NSUnionRange(_InvalidAttributesRange, aRange);
        else
            _InvalidAttributesRange = aRange;
    if(_InvalidAttributesRange.length)
    {
        NSEnumerator * enumerator = [[self layoutManagers] objectEnumerator];
        NSLayoutManager * LM;
        // don't invalidate layout: infinite loop
//        [self invalidateAttributesInRange: _InvalidAttributesRange];
        while(LM = [enumerator nextObject])
            [LM invalidateLayoutForCharacterRange: _InvalidAttributesRange isSoft: NO actualCharacterRange: nil];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= fixAttributesInRange:
- (void) fixAttributesInRange: (NSRange) aRange;
/*"Fix the attributes in the invalid part of the specified range. This is where performance occurs.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    NSLog(@"fixAttributesInRange: %@, %@", NSStringFromRange(aRange), NSStringFromRange(_InvalidAttributesRange));
    [self refixAttributesInRange: NSIntersectionRange(aRange, _InvalidAttributesRange)];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= refixAttributesInRange:
- (void) refixAttributesInRange: (NSRange) aRange;
/*"Fix the attributes in the specified range even if the range is not tagged as invalid.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    NSLog(@"refixAttributesInRange: %@, %@", NSStringFromRange(aRange), NSStringFromRange(_InvalidAttributesRange));
    if(aRange.length)
    {
//        NSLog(@"COUCOUROUCOUCOU");
        [[self syntaxParser] fixAttributesInRange: aRange];
//        NSLog(@"CUCURUCUCU %@", [self syntaxParser]);
        if(_InvalidAttributesRange.length)
            if(NSLocationInRange(_InvalidAttributesRange.location, aRange))
            {
                unsigned max = NSMaxRange(aRange);    
                _InvalidAttributesRange = NSIntersectionRange(_InvalidAttributesRange, NSMakeRange(max, [self length] - max));
            }
            else if(NSLocationInRange(NSMaxRange(_InvalidAttributesRange), aRange))
            {
                _InvalidAttributesRange = NSIntersectionRange(_InvalidAttributesRange, NSMakeRange(0, aRange.location));
            }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= doubleClickAtIndex:
- (NSRange) doubleClickAtIndex: (unsigned) index;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: implement some kind of balance range for range
"*/
{
    return [[self settings] boolForKey: iTMUDSmartSelectionKey]?
        [self smartDoubleClickAtIndex: index]:
        [_MAS doubleClickAtIndex: index];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  smartDoubleClickAtIndex:
- (NSRange) smartDoubleClickAtIndex: (unsigned) index;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * string = [self string];
    if(NSLocationInRange(index, NSMakeRange(0, [string length])))
    {
        BOOL escaped = YES;
        if([string isBackslashAtIndex: index escaped: &escaped])
        {
            if(!escaped && index+1<[string length])
            {
                NSRange result = [_MAS doubleClickAtIndex: index+1];//[self doubleClickAtIndex: index+1 isSmart: NO];
                --result.location;
                ++result.length;
                return result;
            }
            return NSMakeRange(index, 1);
        }
        //else
        switch([string characterAtIndex: index])
        {
            NSRange R;
            case '{':
            case '}':
                if(R = [string groupRangeAtIndex: index beginDelimiter: '{' endDelimiter: '}'], R.length>0)
                    return R;
                break;
            case '(':
            case ')':
                if (R = [string groupRangeAtIndex: index beginDelimiter: '(' endDelimiter: ')'], R.length>0)
                    return R;
                break;
            case '[':
            case ']':
                if (R = [string groupRangeAtIndex: index beginDelimiter: '[' endDelimiter: ']'], R.length>0)
                    return R;
                break;
            case '%':
            {
                BOOL escaped;
                if((index>0) && [string isBackslashAtIndex: index-1 escaped: &escaped] && !escaped)
                    return NSMakeRange(index-1, 2);
                else
                {
                    unsigned int start;
                    unsigned int end;
                    unsigned int contentsEnd;
                    [string getLineStart: &start end: &end contentsEnd: &contentsEnd forRange: NSMakeRange(index, 0)];
                    return (start<index)? NSMakeRange(index, contentsEnd - index): NSMakeRange(start, end - start);
                }
            }
            case '^':
            case '_':
            {
                BOOL escaped;
                if((index+1<[string length]) && (![string isBackslashAtIndex: index-1 escaped: &escaped] || escaped))
                {
                    NSRange R = [string groupRangeAtIndex: index+1];
                    if(R.length)
                    {
                        --R.location;
                        ++R.length;
                        return R;
                    }
                    else
                        return NSMakeRange(index, 1);
                }
            }
        }
        return [_MAS doubleClickAtIndex: index];
    }
    return NSMakeRange(NSNotFound, 0);
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMTextStorage

